/*
 * kprofwidget.h
 *
 * $Id: kprofwidget.h,v 1.11 2001/02/14 14:07:18 fpillet Exp $
 *
 * Copyright (c) 2000-2001 Florent Pillet <fpillet@users.sourceforge.net>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.trolltech.com/
 *
 * Requires the K Desktop Environment 2.0 (KDE 2.0) libraries or later,
 * available at no cost at http://www.kde.org/
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __KPROFWIDGET_H__
#define __KPROFWIDGET_H__

#include <qvector.h>
#include <qglobal.h>
#include <qstring.h>
#include <qlineedit.h>
#include <qprinter.h>
#include <qtextstream.h>

#include <klistview.h>
#include <ktabctl.h>
#include <kurl.h>
#include <kprocess.h>

#include "cprofileinfo.h"

class QFont;
class QFile;
class CProfileViewItem;

class KProfWidget : public QWidget
{
	Q_OBJECT

public:
	static QFont*			sListFont;	// font used to draw list entries

protected:
	KTabCtl*				mTabs;		// the tabbed control
	KListView*				mFlat;		// the flat profile list widget
	KListView*				mHier;		// the hierarchical profile widget
	KListView*				mObjs;		// the object profile widget

	QVector<CProfileInfo>	mProfile;	// profile information read from file
	QVector<QString>		mClasses;	// list of distinct class names found in the profile information

#ifndef QT_NO_PRINTER
	QPrinter				mPrinter;	// printer object
#endif
	int						mCurPage;	// id of the current page (used to know which to print)

	QString					mGProfStdout;	// stdout from gprof command
	QString					mGProfStderr;	// stderr from gprof command
	QString					mFlatFilter; // filter string for flat profile view

	QFont					mListFont;	// font used to draw the text

	bool					mAbbrevTemplates;	// if true, templates are "abbreviates" (i.e. become <...>)

	// this structure is used while parsing the call graph
	typedef struct
	{
		QString	name;
		long	line;
		bool	primary;
		bool	recursive;
	} SCallGraphEntry;

	enum								// states while parsing the gprof output
	{
		SEARCH_FLAT_PROFILE = 0,
		PROCESS_FLAT_PROFILE,
		SEARCH_CALL_GRAPH,
		PROCESS_CALL_GRAPH
	};

public:
	enum colID
	{
		col_function = 0,
		col_recursive,
		col_count,
		col_total,
		col_totalPercent,
		col_self,
		col_totalPerCall,
		col_selfPerCall
	};

public:
	KProfWidget (QWidget *parent=NULL, const char *name=NULL);
	~KProfWidget ();

public slots:
	void tabSelected (int page);

	void settingsChanged ();
	void loadSettings ();
	void applySettings ();

	void openResultsFile ();
	void openRecentFile (const KURL& url);
	void doPrint ();

	void profileEntryRightClick (QListViewItem *listItem, const QPoint &p, int);
	void flatProfileFilterChanged (const QString &filter);
	void generateCallGraph ();

	void toggleTemplateAbbrev ();
	void selectListFont ();

protected slots:
	void selectionChanged (QListViewItem *item);
	void gprofStdout (KProcess*, char *buffer, int buflen);
	void gprofStderr (KProcess*, char *buffer, int buflen);

signals:
	void addRecentFile (const KURL&);

private:
	void openFile (const QString &filename);
	void prepareProfileView (KListView *view, bool rootIsDecorated);
	void parseProfile (QTextStream &t);
	void processCallGraphBlock (const QVector<SCallGraphEntry> &data);
	void postProcessProfile ();

	CProfileInfo *locateProfileEntry (const QString& name);

	void fillFlatProfileList ();
	void fillHierProfileList ();
	void fillHierarchy (CProfileViewItem *item, CProfileInfo *parent, QArray<CProfileInfo *> &addedEntries, int &count);
	void fillObjsProfileList ();

	void selectProfileItem (CProfileInfo *info);
	void selectItemInView (QListView *view, CProfileInfo *info, bool examineSubs);

	void markForOutput (CProfileInfo *info);
	void generateDotCallGraph (QFile& file, bool currentSelectionOnly);
	void generateVCGCallGraph (QFile& file, bool currentSelectionOnly);

	QString getClassName (const QString& name);
	QString removeTemplates (const QString& name);
};

#endif
